@@ -16,8 +16,6 @@
  basic checker for Python code
 """
 
-__revision__ = "$Id: base.py,v 1.65 2006-01-26 00:26:23 syt Exp $"
-
 from logilab import astng
 from logilab.common.ureports import Table
 
@@ -40,6 +38,7 @@ NO_REQUIRED_DOC_RGX = re.compile('__.*__')
 del re
 
 def in_loop(node):
+    """return True if the node is inside a kind of for loop"""
     parent = node.parent
     while parent is not None:
         if isinstance(parent, (astng.For, astng.ListComp, astng.GenExpr)):
@@ -113,6 +112,14 @@ MSGS = {
     'W0104': ('Statement seems to have no effect',
               'Used when a statement doesn\'t have (or at least seems to) \
               any effect.'),
+    'W0105': ('String statement has no effect',
+              'Used when a string is used as a statement (which of course \
+              has no effect). This is a particular case of W0104 with its \
+              own message so you can easily disable it if you\'re using \
+              those strings as documentation, instead of comments.'),
+    'W0106': ('Unnecessary semi-column',
+              'Used when a statement is endend by a semi-colon (";"), which \
+              isn\'t necessary (that\'s python, not C ;)'),
 
     'W0122': ('Use of the exec statement',
               'Used when you use the "exec" statement, to discourage its \
@@ -166,7 +173,7 @@ functions, methods
     msgs = MSGS
     priority = -1
     options = (('required-attributes',
-                {'default' : ('__revision__',), 'type' : 'csv',
+                {'default' : (), 'type' : 'csv',
                  'metavar' : '<attributes>',
                  'help' : 'Required attributes for module, separated by a '
                           'comma'}
@@ -303,8 +310,25 @@ functions, methods
             self._check_name('attr', attr, anodes[0])
 
     def visit_discard(self, node):
-        """check for statement without effect"""
-        if not isinstance(node.expr, astng.CallFunc):
+        """check for various kind of statements without effect"""
+        expr = node.expr
+        if isinstance(node.expr, astng.Const):
+            # XXX lineno maybe dynamically set incidently
+            if expr.value is None and expr.lineno is None:
+                # const None node with lineno to None are inserted
+                # on unnecessary semi-column
+                # XXX navigate to get a correct lineno
+                brothers = list(node.parent.getChildNodes())
+                previoussibling = brothers[brothers.index(node)-1]
+                self.add_message('W0106', node=previoussibling)
+                return
+            if isinstance(expr.value, basestring):
+                # tread string statement in a separated message
+                self.add_message('W0105', node=node)
+                return
+        # ignore if this is a function call (can't predicate side effects)
+        # or a yield (which are wrapped by a discard node in py >= 2.5)
+        if not isinstance(node.expr, (astng.CallFunc, astng.Yield)):
             self.add_message('W0104', node=node)
 
     def visit_function(self, node):
