@@ -16,7 +16,7 @@
 """imports checkers for Python code
 """
 
-__revision__ = "$Id: imports.py,v 1.46 2005/11/10 17:26:27 syt Exp $"
+__revision__ = "$Id: imports.py,v 1.46 2005-11-10 17:26:27 syt Exp $"
 
 from logilab.common import get_cycles
 from logilab.common.modutils import is_standard_module, is_relative, \
@@ -228,8 +228,10 @@ given file (report R0402 must not be disabled)'}
         
     def close(self):
         """called before visiting project (i.e set of modules)"""
-        for cycle in get_cycles(self.import_graph):
-            self.add_message('R0401', args=' -> '.join(cycle))
+        # don't try to compute cycles if the associated message is disabled
+        if self.linter.is_message_enabled('R0401'):
+            for cycle in get_cycles(self.import_graph):
+                self.add_message('R0401', args=' -> '.join(cycle))
          
     def visit_import(self, node):
         """triggered when an import statement is seen"""
