@@ -1,4 +1,4 @@
-# pylint:disable-msg=W0212
+# pylint:disable-msg=R0201
 """docstring"""
 __revision__ = ''
 
@@ -72,3 +72,28 @@ class BadArgument:
         """docstring"""
         pass
     
+class InterfaceCantBeFound:
+    """docstring"""
+    __implements__ = undefined
+
+    def __init__(self):
+        """only to make pylint happier"""
+    
+    def please(self):
+        """public method 1/2"""
+
+    def besilent(self):
+        """public method 2/2"""
+
+class InterfaceCantBeFound2:
+    """docstring"""
+    __implements__ = BadArgument.__implements__ + Correct2.__implements__
+
+    def __init__(self):
+        """only to make pylint happier"""
+    
+    def please(self):
+        """public method 1/2"""
+
+    def besilent(self):
+        """public method 2/2"""
