@@ -1,4 +1,4 @@
-#pylint: disable-msg=W0131,C0321
+#pylint: disable-msg=C0111,C0321
 """pylint complains about 'index' being used before definition"""
 
 __revision__ = None
@@ -7,7 +7,7 @@ print [index
        for index in range(10)]
 
 
-filter_func = lambda x: not x
+FILTER_FUNC = lambda x: not x
 
 def func(xxx): return xxx
 
@@ -17,4 +17,4 @@ import sys; print sys.exc_info( )
 
 for i in range(10): print i
 
-j = 4; lamb = lambda x: x+j
+j = 4; LAMB = lambda x: x+j
